@@ -1,7 +1,7 @@
 /*
  * sma -- Sendmail log analyser
  *
- * Copyright (c) 2000, 2001, 2002 Jarkko Turkulainen. All rights reserved.
+ * Copyright (c) 2000 - 2003 Jarkko Turkulainen. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Date: 2002/09/18 14:20:37 $
+ * $Date: 2003/03/22 15:30:37 $
  */
 
 #include "sma.h"
@@ -435,7 +435,7 @@ html(FILE *fp) {
 			"        <TD ALIGN=center BGCOLOR=\"#%s\">"
 			"%%</TD>\n"
 			"      </TR>\n", tbchar, tbchar, tbchar, tbchar, tbchar);
-		for (j = 0; j < (MIN(lnum, hptr->rrdif)); j++) {
+		for (j = 0; j < (MIN(rpnum, hptr->rrdif)); j++) {
 			fprintf(fp, "      <TR>\n");
 			fprintf(fp, "        <TD ALIGN=center >%d</TD>\n", j+1);
 			fprintf(fp, "        <TD>%s<BR>%s</TD>\n", hptr->srtab[j]->fname, hptr->srtab[j]->tname);
@@ -751,7 +751,7 @@ html(FILE *fp) {
 	if (ftchar)
 			fprintf(fp, "    <I>%s</I>\n", ftchar);
 		else
-			fprintf(fp, "    <I>Copyright (c) 2000-2002"
+			fprintf(fp, "    <I>Copyright (c) 2000 - 2003"
 				" Jarkko Turkulainen. All rights reserved.</I>\n");
 	fprintf(fp, "  </BODY>\n</HTML>\n");
 }
