@@ -1,7 +1,7 @@
 /*
  * sma -- Sendmail log analyser
  *
- * Copyright (c) 2000, 2001, 2002 Jarkko Turkulainen. All rights reserved.
+ * Copyright (c) 2000 - 2003 Jarkko Turkulainen. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Date: 2002/11/28 10:35:33 $
+ * $Date: 2003/04/27 15:43:33 $
  */
 
 #include "sma.h"
@@ -88,9 +88,10 @@ parse(FILE *fp, const char *file) {
 
 	/* collecting */
 	char *action, *dsn, *idptr;
+	char *messageid = NULL;
 	const char *tmphost;
 	int logformat, hcount, icount;
-	int status = 0, idsize = 0, gf, pf = 0, actn = 0; 
+	int status = 0, idsize = 0, gf, pf = 0, actn = 0, sunos = 0; 
 	int actflag, year, mon, day, hh, min, sec;
 	struct msgid *tmsgid = NULL;
 
@@ -171,6 +172,7 @@ parse(FILE *fp, const char *file) {
 					  "file %s\n", pname, lcount, file);
 				continue;
 			}
+			/* We assume things fixed with NT log.. */
 			if (hcount >= 6) {
 				idptr = head[4];
 				action = head[5];
@@ -179,8 +181,6 @@ parse(FILE *fp, const char *file) {
 			}
 		/*
 		 * Normal syslog
-		 * This also include SunOS 5.x ID generation tags
-		 * (see the hack below)..
 		 */
 		} else {
 			logformat = 0;
@@ -199,29 +199,42 @@ parse(FILE *fp, const char *file) {
 			}
 			day = atoi(head[1]);
 			mon = conv_mon(head[0]);
-
-			/* 
-			 * SunOS 5.x with ID generation enabled:
-			 * Somewhat stupid hack, might be better..
-			 */
+			
+			/* SunOS 5.x with ID generation enabled: */
 			if (!strncmp("[ID", head[5], 3)) {
-				if (hcount >= 8) idptr = head[8];
-				else continue;
-				if (hcount >= 10) {
-					action = head[9];
-					actn = 9; dsn = head[10];
-				}
+				sunos = 3;
+			} else sunos = 0;
+			/* 
+			 * Now, we run through the first info-vector (head)
+			 * and try to find useful keywords. If found, copy
+			 * the pointers as idptr, action and dsn. Action
+			 * flags are later set according to these keywords.
+			 */
+			for (k = 6+sunos; k < hcount; k++ ) {
+				/* "normal" delivery lines */
+				if (!strncmp("daemon", head[k], 6) ||
+				    !strncmp("ruleset", head[k], 7) || 
+				    !strncmp("to=", head[k], 3) || 
+				    !strncmp("from=", head[k], 5) || 
+				    !strncmp("SYSERR", head[k], 6) || 
+				    !strncmp("database", head[k], 8)) {
+					actn = k;
+					idptr = head[k-1];
+					action = head[k];
+					break;
 
-			} else {
-				if (hcount >= 5) idptr = head[5];
-				else continue;
-				if (hcount >= 7) {
-					action = head[6];
-					actn = 6; dsn = head[7];
+				/* DSN, and other strange stuff: */
+				} else if (!strncmp("DSN", head[k], 3) ||
+				    !strncmp("User", head[k], 4) || 
+				    !strncmp("postmaster", head[k],10) || 
+				    !strncmp("return", head[k], 6)) {
+					idptr = head[k-2];
+					action = head[k-1];
+					dsn = head[k];
+					break;
 				}
-			} 
+			}
 		}
-
 		/* 
 		 * Permitted actions are defined here.
 		 *
@@ -248,9 +261,7 @@ parse(FILE *fp, const char *file) {
 			actflag = ACTION_DATAB;
 		} else if (dsn && (!strncmp("User", dsn, 4))) {
 			actflag = ACTION_USER;
-		} else if (dsn && (!strncmp("DSN", dsn, 3) ||
-			    !strncmp("postmaster", dsn, 10) ||
-			    !strncmp("return", dsn, 6))) {
+		} else if (dsn) {
 			actflag = ACTION_DSN;
 		} else continue;
 
@@ -353,7 +364,7 @@ parse(FILE *fp, const char *file) {
 #endif
 				/* Message ID */
 				update_msgid(hptr, p, s, action, hh, 
-					day, 1, 200);
+					day, 1, 200, "<DSN>");
 			/* Nope, we are filtered: */
 			} else
 				continue;
@@ -386,6 +397,10 @@ parse(FILE *fp, const char *file) {
 				if (!strncmp("size", info[k], 4)) {
 					hptr->lsize = atoi(info[k]+5);
 
+				/* msgid */
+				} else if (!strncmp("msgid", info[k], 5)) {
+					messageid = info[k]+6;
+
 				/* input relay */
 				} else if (!strncmp("relay", info[k], 5)) {
 					s = get_relay_name(info[k]+6);
@@ -394,9 +409,13 @@ parse(FILE *fp, const char *file) {
 				} else if (!strncmp("nrcpts", info[k], 6))
 					idsize = atoi(info[k]+7);
 			}
+			/* Check relay */
 			if (!s) s = BOUNCE_HOST;
 			if (vflag) fprintf(stderr, "  relay=%s\n", s);
 
+			/* Initialize messageid */
+			if (!messageid) messageid = "";
+
 			/*
 			 * Sender Envelope and Relay filtering
 			 */
@@ -407,7 +426,7 @@ parse(FILE *fp, const char *file) {
 			if (check_filter(sef, p) && check_filter(srf, s)) {
 #endif
 				update_msgid(hptr, p, s, idptr, hh, day,
-					idsize, hptr->lsize);
+					idsize, hptr->lsize, messageid);
 			/* Nope, we are filtered: */
 			} else
 				continue;
@@ -448,7 +467,8 @@ parse(FILE *fp, const char *file) {
 			for (; k < icount; k++ ) {
 				if (!strncmp("stat", info[k], 4)) {
 					statstr = info[k]+5;
-					if (!strncmp("Sent", info[k]+5, 4))
+					if (!strncmp("Sent", info[k]+5, 4) ||
+					    !strncmp("sent", info[k]+5, 4))
 						status = STATUS_SENT;
 					else if (!strncmp("Deferred", 
 					  info[k]+5, 8)) 
@@ -529,6 +549,7 @@ parse(FILE *fp, const char *file) {
 							hptr->cdtime,
 							tmphost,
 							(int)hptr->lsize,
+							tmsgid->msgid,
 							tmsgid->sender,
 							tmsgid->relay,
 							env_rec[i], s,
@@ -738,7 +759,14 @@ parse(FILE *fp, const char *file) {
 const char *
 get_name(char *name) {
 	char *tmp, *tmp2;
+	int i;
 
+	/* downcase everything if requested */
+	if (dcaddrflag) {
+		for ( i=0; name[i]; i++) {
+			name[i] = tolower(name[i]);
+		}
+	}
 	/* null-sender? */
 	if (!strncmp("<>", name, 2))
 		return("<>");
@@ -855,8 +883,9 @@ check_filter(const char *s, const char *text) {
  * Printing routine for Custom Log format
  */
 void
-printclog(time_t sma_time, const char *host, int size, const char *from,
-	 const char *frelay, const char *to, const char *trelay, int status) {
+printclog(time_t sma_time, const char *host, int size, const char *id,
+	const char *from, const char *frelay, const char *to,
+	const char *trelay, int status) {
 
 	/* time struct: */
 	struct tm *tmptime;
@@ -918,6 +947,10 @@ printclog(time_t sma_time, const char *host, int size, const char *from,
 							fprintf(ofp,"%s", host);
 							frm++;
 							break;
+						case 'i':
+							fprintf(ofp,"%s", id);
+							frm++;
+							break;
 						case 'm':
 							fprintf(ofp,"%.2d",
 							  tmptime->tm_mon+1);
